#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.uzbekistan import Uzbekistan
from tests.common import CommonCountryTests, WorkingDayTests


class TestUzbekistan(CommonCountryTests, WorkingDayTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Uzbekistan)

    def test_special_holidays(self):
        self.assertHoliday(
            "2018-01-02",
            "2018-03-20",
            "2018-08-31",
            "2019-01-02",
            "2019-03-22",
            "2019-09-02",
            "2020-03-23",
            "2021-05-14",
            "2021-09-02",
            "2021-09-03",
            "2021-12-31",
            "2022-01-03",
            "2022-03-22",
            "2022-03-23",
            "2022-05-03",
            "2022-07-11",
            "2022-09-02",
            "2023-01-02",
            "2023-04-24",
            "2023-06-29",
            "2023-06-30",
            "2024-03-22",
            "2024-04-11",
            "2024-06-18",
            "2024-09-03",
            "2024-12-31",
            "2025-12-31",
            "2026-01-02",
            "2026-05-28",
            "2026-05-29",
            "2026-08-31",
        )

    def test_substituted_holidays(self):
        self.assertHoliday(
            "2018-01-03",
            "2018-03-19",
            "2018-03-22",
            "2018-08-23",
            "2018-08-24",
            "2018-09-03",
            "2018-09-04",
            "2018-12-31",
            "2019-01-03",
            "2019-06-06",
            "2019-09-03",
            "2019-12-31",
            "2020-01-02",
            "2020-08-31",
            "2021-03-22",
            "2021-07-21",
            "2021-07-22",
            "2022-01-04",
            "2022-05-04",
            "2022-07-12",
            "2023-01-03",
            "2023-03-20",
            "2023-03-22",
            "2024-01-02",
            "2024-04-12",
            "2024-12-30",
            "2025-01-02",
            "2026-12-31",
        )

    def test_workdays(self):
        self.assertWorkingDay(
            "2018-01-06",
            "2018-03-17",
            "2018-03-24",
            "2018-08-25",
            "2018-08-26",
            "2018-09-08",
            "2018-09-15",
            "2018-12-29",
            "2019-01-05",
            "2019-06-01",
            "2019-09-07",
            "2019-12-28",
            "2020-01-04",
            "2020-08-29",
            "2021-03-27",
            "2021-07-17",
            "2021-07-24",
            "2022-01-08",
            "2022-05-07",
            "2022-07-16",
            "2023-01-07",
            "2023-03-11",
            "2023-03-25",
            "2024-01-06",
            "2024-04-13",
            "2024-12-14",
            "2025-01-04",
            "2026-12-12",
        )

    def test_new_years_day(self):
        self.assertHolidayName("Yangi yil", (f"{year}-01-01" for year in self.full_range))

    def test_womens_day(self):
        self.assertHolidayName("Xotin-qizlar kuni", (f"{year}-03-08" for year in self.full_range))

    def test_nowruz(self):
        self.assertHolidayName("Navroʻz bayrami", (f"{year}-03-21" for year in self.full_range))

    def test_memory_and_honor_day(self):
        name_1992 = "Gʻalaba kuni"
        name_1999 = "Xotira va qadrlash kuni"
        self.assertHolidayName(
            name_1992, (f"{year}-05-09" for year in range(self.start_year, 1999))
        )
        self.assertHolidayName(name_1999, (f"{year}-05-09" for year in range(1999, self.end_year)))
        self.assertNoHolidayName(name_1992, range(1999, self.end_year))
        self.assertNoHolidayName(name_1999, range(self.start_year, 1999))

    def test_independence_day(self):
        self.assertHolidayName("Mustaqillik kuni", (f"{year}-09-01" for year in self.full_range))

    def test_teachers_and_instructors_day(self):
        name = "Oʻqituvchi va murabbiylar kuni"
        self.assertHolidayName(name, (f"{year}-10-01" for year in range(1997, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1997))

    def test_constitution_day(self):
        name = "Oʻzbekiston Respublikasi Konstitutsiyasi kuni"
        self.assertHolidayName(name, (f"{year}-12-08" for year in range(1993, self.end_year)))
        self.assertNoHolidayName(name, 1992)

    def test_eid_al_fitr(self):
        name = "Roʻza hayit"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Qurbon hayit"
        self.assertHolidayName(
            name,
            "2006-01-10",
            "2006-12-30",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_observed(self):
        obs_dts = (
            "2023-10-02",
            "2024-06-17",
            "2024-09-02",
            "2024-12-09",
            "2025-03-31",
        )
        self.assertHoliday(obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2020(self):
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "Yangi yil"),
            ("2020-01-02", "Dam olish kuni (04/01 2020 dan koʻchirilgan)"),
            ("2020-03-08", "Xotin-qizlar kuni"),
            ("2020-03-21", "Navroʻz bayrami"),
            ("2020-03-23", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2020-05-09", "Xotira va qadrlash kuni"),
            ("2020-05-24", "Roʻza hayit"),
            ("2020-07-31", "Qurbon hayit"),
            ("2020-08-31", "Dam olish kuni (29/08 2020 dan koʻchirilgan)"),
            ("2020-09-01", "Mustaqillik kuni"),
            ("2020-10-01", "Oʻqituvchi va murabbiylar kuni"),
            ("2020-12-08", "Oʻzbekiston Respublikasi Konstitutsiyasi kuni"),
        )

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Yangi yil"),
            ("2021-03-08", "Xotin-qizlar kuni"),
            ("2021-03-21", "Navroʻz bayrami"),
            ("2021-03-22", "Dam olish kuni (27/03 2021 dan koʻchirilgan)"),
            ("2021-05-09", "Xotira va qadrlash kuni"),
            ("2021-05-13", "Roʻza hayit"),
            ("2021-05-14", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2021-07-20", "Qurbon hayit"),
            ("2021-07-21", "Dam olish kuni (17/07 2021 dan koʻchirilgan)"),
            ("2021-07-22", "Dam olish kuni (24/07 2021 dan koʻchirilgan)"),
            ("2021-09-01", "Mustaqillik kuni"),
            ("2021-09-02", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2021-09-03", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2021-10-01", "Oʻqituvchi va murabbiylar kuni"),
            ("2021-12-08", "Oʻzbekiston Respublikasi Konstitutsiyasi kuni"),
            ("2021-12-31", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Yangi yil"),
            ("2022-01-03", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2022-01-04", "Dam olish kuni (08/01 2022 dan koʻchirilgan)"),
            ("2022-03-08", "Xotin-qizlar kuni"),
            ("2022-03-21", "Navroʻz bayrami"),
            ("2022-03-22", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2022-03-23", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2022-05-02", "Roʻza hayit"),
            ("2022-05-03", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2022-05-04", "Dam olish kuni (07/05 2022 dan koʻchirilgan)"),
            ("2022-05-09", "Xotira va qadrlash kuni"),
            ("2022-07-09", "Qurbon hayit"),
            ("2022-07-11", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2022-07-12", "Dam olish kuni (16/07 2022 dan koʻchirilgan)"),
            ("2022-09-01", "Mustaqillik kuni"),
            ("2022-09-02", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2022-10-01", "Oʻqituvchi va murabbiylar kuni"),
            ("2022-12-08", "Oʻzbekiston Respublikasi Konstitutsiyasi kuni"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Yangi yil"),
            ("2023-01-02", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2023-01-03", "Dam olish kuni (07/01 2023 dan koʻchirilgan)"),
            ("2023-03-08", "Xotin-qizlar kuni"),
            ("2023-03-20", "Dam olish kuni (11/03 2023 dan koʻchirilgan)"),
            ("2023-03-21", "Navroʻz bayrami"),
            ("2023-03-22", "Dam olish kuni (25/03 2023 dan koʻchirilgan)"),
            ("2023-04-21", "Roʻza hayit"),
            ("2023-04-24", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2023-05-09", "Xotira va qadrlash kuni"),
            ("2023-06-28", "Qurbon hayit"),
            ("2023-06-29", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2023-06-30", "Prezidentining farmoni bilan qoʻshimcha dam olish kuni"),
            ("2023-09-01", "Mustaqillik kuni"),
            ("2023-10-01", "Oʻqituvchi va murabbiylar kuni"),
            ("2023-10-02", "Oʻqituvchi va murabbiylar kuni (koʻchirilgan)"),
            ("2023-12-08", "Oʻzbekiston Respublikasi Konstitutsiyasi kuni"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "Additional day off by Presidential decree"),
            ("2023-01-03", "Day off (substituted from 01/07/2023)"),
            ("2023-03-08", "Women's Day"),
            ("2023-03-20", "Day off (substituted from 03/11/2023)"),
            ("2023-03-21", "Nowruz"),
            ("2023-03-22", "Day off (substituted from 03/25/2023)"),
            ("2023-04-21", "Eid al-Fitr"),
            ("2023-04-24", "Additional day off by Presidential decree"),
            ("2023-05-09", "Day of Memory and Honor"),
            ("2023-06-28", "Eid al-Adha"),
            ("2023-06-29", "Additional day off by Presidential decree"),
            ("2023-06-30", "Additional day off by Presidential decree"),
            ("2023-09-01", "Independence Day"),
            ("2023-10-01", "Teachers and Instructors Day"),
            ("2023-10-02", "Teachers and Instructors Day (observed)"),
            ("2023-12-08", "Constitution Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-01-02", "Додатковий вихідний згідно указу Президента"),
            ("2023-01-03", "Вихідний день (перенесено з 07.01.2023)"),
            ("2023-03-08", "Жіночий день"),
            ("2023-03-20", "Вихідний день (перенесено з 11.03.2023)"),
            ("2023-03-21", "Свято Новруз"),
            ("2023-03-22", "Вихідний день (перенесено з 25.03.2023)"),
            ("2023-04-21", "Рамазан-байрам"),
            ("2023-04-24", "Додатковий вихідний згідно указу Президента"),
            ("2023-05-09", "День памʼяті і шани"),
            ("2023-06-28", "Курбан-байрам"),
            ("2023-06-29", "Додатковий вихідний згідно указу Президента"),
            ("2023-06-30", "Додатковий вихідний згідно указу Президента"),
            ("2023-09-01", "День незалежності"),
            ("2023-10-01", "День вчителя і наставника"),
            ("2023-10-02", "День вчителя і наставника (вихідний)"),
            ("2023-12-08", "День Конституції Республіки Узбекистан"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.angola import Angola
from tests.common import CommonCountryTests


class TestAngola(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Angola, years_non_observed=range(2000, 2030))

    def test_special_holidays(self):
        self.assertHoliday("2017-08-23")

    def test_new_years_day(self):
        name = "Dia do Ano Novo"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2006-01-02",
            "2018-12-31",
            "2026-01-02",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_martyrs_of_colonial_repression_day(self):
        name = "Dia dos Mártires da Repressão Colonial"
        self.assertHolidayName(name, (f"{year}-01-04" for year in range(1997, 2012)))
        self.assertNoHolidayName(name, range(self.start_year, 1997), range(2012, self.end_year))
        obs_dts = (
            "2004-01-05",
            "2009-01-05",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_armed_struggle_day(self):
        name_1975 = "Dia do Início da Luta Armada"
        name_2012 = "Dia do Início da Luta Armada de Libertação Nacional"
        self.assertHolidayName(
            name_1975, (f"{year}-02-04" for year in range(self.start_year, 2012))
        )
        self.assertHolidayName(name_2012, (f"{year}-02-04" for year in range(2012, self.end_year)))
        self.assertNoHolidayName(name_1975, range(2012, self.end_year))
        self.assertNoHolidayName(name_2012, range(self.start_year, 2012))
        obs_dts = (
            "2001-02-05",
            "2007-02-05",
        )
        self.assertHolidayName(f"{name_1975} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        obs_dts = (
            "2018-02-05",
            "2020-02-03",
            "2021-02-05",
            "2025-02-03",
        )
        self.assertHolidayName(f"{name_2012} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_carnival_day(self):
        name = "Dia do Carnaval"
        self.assertHolidayName(
            name,
            "1997-02-11",
            "2000-03-07",
            "2010-02-16",
            "2018-02-13",
            "2019-03-05",
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
        )
        self.assertHolidayName(name, range(1997, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1997))

        obs_dts = (
            "2019-03-04",
            "2020-02-24",
            "2021-02-15",
            "2022-02-28",
            "2023-02-20",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_womens_day(self):
        name = "Dia Internacional da Mulher"
        self.assertHolidayName(name, (f"{year}-03-08" for year in range(1997, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1997))
        obs_dts = (
            "2009-03-09",
            "2015-03-09",
            "2022-03-07",
            "2029-03-09",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_southern_africa_liberation_day(self):
        name = "Dia da Libertação da África Austral"
        self.assertHolidayName(name, (f"{year}-03-23" for year in range(2019, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2019))
        obs_dts = (
            "2021-03-22",
            "2023-03-24",
            "2027-03-22",
            "2028-03-24",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_peace_and_national_reconciliation_day(self):
        name = "Dia da Paz e Reconciliação Nacional"
        self.assertHolidayName(name, (f"{year}-04-04" for year in range(2003, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2003))
        obs_dts = (
            "2004-04-05",
            "2010-04-05",
            "2019-04-05",
            "2023-04-03",
            "2024-04-05",
            "2028-04-03",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Sexta-Feira Santa"
        self.assertHolidayName(
            name,
            "1997-03-28",
            "2000-04-21",
            "2010-04-02",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        self.assertHolidayName(name, range(1997, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1997))

    def test_international_workers_day(self):
        name = "Dia Internacional do Trabalhador"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2025-05-02",
            "2029-04-30",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_africa_day(self):
        name = "Dia da África"
        self.assertHolidayName(name, (f"{year}-05-25" for year in range(2001, 2011)))
        self.assertNoHolidayName(name, range(self.start_year, 2001), range(2011, self.end_year))
        obs_dts = (
            "2003-05-26",
            "2008-05-26",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_childrens_day(self):
        name = "Dia Internacional da Criança"
        self.assertHolidayName(name, (f"{year}-06-01" for year in range(1997, 2011)))
        self.assertNoHolidayName(name, range(self.start_year, 1997), range(2011, self.end_year))
        obs_dts = (
            "2003-06-02",
            "2008-06-02",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_heroes_day(self):
        name = "Dia do Fundador da Nação e do Herói Nacional"
        self.assertHolidayName(name, (f"{year}-09-17" for year in range(1980, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1980))
        obs_dts = (
            "2000-09-18",
            "2006-09-18",
            "2017-09-18",
            "2019-09-16",
            "2020-09-18",
            "2024-09-16",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_all_souls_day(self):
        name = "Dia dos Finados"
        self.assertHolidayName(name, (f"{year}-11-02" for year in self.full_range))
        obs_dts = (
            "2003-11-03",
            "2008-11-03",
            "2021-11-01",
            "2023-11-03",
            "2027-11-01",
        )
        self.assertHolidayName(f"{name} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_independence_day(self):
        name_1975 = "Dia da Independência"
        name_1996 = "Dia da Independência Nacional"
        self.assertHolidayName(
            name_1975, (f"{year}-11-11" for year in range(self.start_year, 1996))
        )
        self.assertHolidayName(name_1996, (f"{year}-11-11" for year in range(1996, self.end_year)))
        self.assertNoHolidayName(name_1975, range(1996, self.end_year))
        self.assertNoHolidayName(name_1996, range(self.start_year, 1996))
        obs_dts = (
            "2001-11-12",
            "2007-11-12",
            "2012-11-12",
            "2021-11-12",
            "2025-11-10",
        )
        self.assertHolidayName(f"{name_1996} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_date_of_founding_of_mpla(self):
        name = "Data da Fundacao do MPLA - Partido do Trabalho"
        self.assertHolidayName(name, (f"{year}-12-10" for year in range(self.start_year, 1992)))
        self.assertNoHolidayName(name, range(1992, self.end_year))

    def test_christmas_and_family_day(self):
        name_1975 = "Dia da Família"
        name_1996 = "Dia do Natal"
        name_2011 = "Dia de Natal e da Família"
        self.assertHolidayName(
            name_1975, (f"{year}-12-25" for year in range(self.start_year, 1996))
        )
        self.assertHolidayName(name_1996, (f"{year}-12-25" for year in range(1996, 2011)))
        self.assertHolidayName(name_2011, (f"{year}-12-25" for year in range(2011, self.end_year)))
        self.assertNoHolidayName(name_1975, range(1996, self.end_year))
        self.assertNoHolidayName(
            name_1996, range(self.start_year, 1996), range(2011, self.end_year)
        )
        self.assertNoHolidayName(name_2011, range(self.start_year, 2011))

        obs_dts = ("2005-12-26",)
        self.assertHolidayName(f"{name_1996} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        obs_dts = (
            "2018-12-24",
            "2025-12-26",
            "2029-12-24",
        )
        self.assertHolidayName(f"{name_2011} (ponte)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Dia do Ano Novo"),
            ("2022-02-04", "Dia do Início da Luta Armada de Libertação Nacional"),
            ("2022-02-28", "Dia do Carnaval (ponte)"),
            ("2022-03-01", "Dia do Carnaval"),
            ("2022-03-07", "Dia Internacional da Mulher (ponte)"),
            ("2022-03-08", "Dia Internacional da Mulher"),
            ("2022-03-23", "Dia da Libertação da África Austral"),
            ("2022-04-04", "Dia da Paz e Reconciliação Nacional"),
            ("2022-04-15", "Sexta-Feira Santa"),
            ("2022-05-01", "Dia Internacional do Trabalhador"),
            ("2022-09-17", "Dia do Fundador da Nação e do Herói Nacional"),
            ("2022-11-02", "Dia dos Finados"),
            ("2022-11-11", "Dia da Independência Nacional"),
            ("2022-12-25", "Dia de Natal e da Família"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Dia do Ano Novo"),
            ("2023-02-04", "Dia do Início da Luta Armada de Libertação Nacional"),
            ("2023-02-20", "Dia do Carnaval (ponte)"),
            ("2023-02-21", "Dia do Carnaval"),
            ("2023-03-08", "Dia Internacional da Mulher"),
            ("2023-03-23", "Dia da Libertação da África Austral"),
            ("2023-03-24", "Dia da Libertação da África Austral (ponte)"),
            ("2023-04-03", "Dia da Paz e Reconciliação Nacional (ponte)"),
            ("2023-04-04", "Dia da Paz e Reconciliação Nacional"),
            ("2023-04-07", "Sexta-Feira Santa"),
            ("2023-05-01", "Dia Internacional do Trabalhador"),
            ("2023-09-17", "Dia do Fundador da Nação e do Herói Nacional"),
            ("2023-11-02", "Dia dos Finados"),
            ("2023-11-03", "Dia dos Finados (ponte)"),
            ("2023-11-11", "Dia da Independência Nacional"),
            ("2023-12-25", "Dia de Natal e da Família"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-02-04", "Liberation Movement Day"),
            ("2023-02-20", "Day off for Carnival Day"),
            ("2023-02-21", "Carnival Day"),
            ("2023-03-08", "International Women's Day"),
            ("2023-03-23", "Southern Africa Liberation Day"),
            ("2023-03-24", "Day off for Southern Africa Liberation Day"),
            ("2023-04-03", "Day off for Peace and National Reconciliation Day"),
            ("2023-04-04", "Peace and National Reconciliation Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-05-01", "International Worker's Day"),
            ("2023-09-17", "National Heroes' Day"),
            ("2023-11-02", "All Souls' Day"),
            ("2023-11-03", "Day off for All Souls' Day"),
            ("2023-11-11", "National Independence Day"),
            ("2023-12-25", "Christmas and Family Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-02-04", "День початку збройної боротьби за національне визволення"),
            ("2023-02-20", "Карнавал (вихідний)"),
            ("2023-02-21", "Карнавал"),
            ("2023-03-08", "Міжнародний жіночий день"),
            ("2023-03-23", "День визволення південної Африки"),
            ("2023-03-24", "День визволення південної Африки (вихідний)"),
            ("2023-04-03", "День миру та національного примирення (вихідний)"),
            ("2023-04-04", "День миру та національного примирення"),
            ("2023-04-07", "Страсна пʼятниця"),
            ("2023-05-01", "Міжнародний день трудящих"),
            ("2023-09-17", "День засновника нації та національного героя"),
            ("2023-11-02", "День усіх померлих"),
            ("2023-11-03", "День усіх померлих (вихідний)"),
            ("2023-11-11", "День національної незалежності"),
            ("2023-12-25", "Різдво Христове та День родини"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.dominican_republic import DominicanRepublic
from tests.common import CommonCountryTests


class TestDominicanRepublic(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(DominicanRepublic)

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany(self):
        name = "Día de los Santos Reyes"
        self.assertNonObservedHolidayName(name, (f"{year}-01-06" for year in self.full_range))
        self.assertHolidayName(
            name,
            "2020-01-06",
            "2021-01-04",
            "2022-01-10",
            "2023-01-09",
            "2024-01-06",
            "2025-01-06",
        )

    def test_lady_of_altagracia(self):
        self.assertHolidayName(
            "Día de la Altagracia", (f"{year}-01-21" for year in self.full_range)
        )

    def test_juan_pablo_duarte_day(self):
        name = "Día de Duarte"
        self.assertNonObservedHolidayName(name, (f"{year}-01-26" for year in self.full_range))
        self.assertHolidayName(
            name,
            "2020-01-26",
            "2021-01-25",
            "2022-01-24",
            "2023-01-30",
            "2024-01-29",
            "2025-01-26",
        )

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de Independencia", (f"{year}-02-27" for year in self.full_range)
        )

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Día del Trabajo"
        self.assertNonObservedHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        self.assertHolidayName(
            name,
            "2020-05-04",
            "2021-05-01",
            "2022-05-02",
            "2023-05-01",
            "2024-04-29",
            "2025-05-05",
        )

    def test_corpus_christi_day(self):
        name = "Corpus Christi"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
        )
        self.assertHolidayName(name, self.full_range)

    def test_restoration_day(self):
        name = "Día de la Restauración"
        self.assertNonObservedHolidayName(name, (f"{year}-08-16" for year in self.full_range))
        self.assertHolidayName(
            name,
            "2020-08-16",
            "2021-08-16",
            "2022-08-15",
            "2023-08-14",
            "2024-08-16",
            "2025-08-16",
        )

    def test_our_lady_of_mercedes_day(self):
        self.assertHolidayName(
            "Día de las Mercedes", (f"{year}-09-24" for year in self.full_range)
        )

    def test_constitution_day(self):
        name = "Día de la Constitución"
        self.assertNonObservedHolidayName(name, (f"{year}-11-06" for year in self.full_range))
        self.assertHolidayName(
            name,
            "2020-11-09",
            "2021-11-06",
            "2022-11-06",
            "2023-11-06",
            "2024-11-04",
            "2025-11-10",
        )

    def test_christmas_day(self):
        self.assertHolidayName("Día de Navidad", (f"{year}-12-25" for year in self.full_range))

    def test_2020(self):
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "Año Nuevo"),
            ("2020-01-06", "Día de los Santos Reyes"),
            ("2020-01-21", "Día de la Altagracia"),
            ("2020-01-26", "Día de Duarte"),
            ("2020-02-27", "Día de Independencia"),
            ("2020-04-10", "Viernes Santo"),
            ("2020-05-04", "Día del Trabajo"),
            ("2020-06-11", "Corpus Christi"),
            ("2020-08-16", "Día de la Restauración"),
            ("2020-09-24", "Día de las Mercedes"),
            ("2020-11-09", "Día de la Constitución"),
            ("2020-12-25", "Día de Navidad"),
        )

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "Año Nuevo"),
            ("2021-01-04", "Día de los Santos Reyes"),
            ("2021-01-21", "Día de la Altagracia"),
            ("2021-01-25", "Día de Duarte"),
            ("2021-02-27", "Día de Independencia"),
            ("2021-04-02", "Viernes Santo"),
            ("2021-05-01", "Día del Trabajo"),
            ("2021-06-03", "Corpus Christi"),
            ("2021-08-16", "Día de la Restauración"),
            ("2021-09-24", "Día de las Mercedes"),
            ("2021-11-06", "Día de la Constitución"),
            ("2021-12-25", "Día de Navidad"),
        )

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Año Nuevo"),
            ("2022-01-10", "Día de los Santos Reyes"),
            ("2022-01-21", "Día de la Altagracia"),
            ("2022-01-24", "Día de Duarte"),
            ("2022-02-27", "Día de Independencia"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-02", "Día del Trabajo"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-08-15", "Día de la Restauración"),
            ("2022-09-24", "Día de las Mercedes"),
            ("2022-11-06", "Día de la Constitución"),
            ("2022-12-25", "Día de Navidad"),
        )

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "Año Nuevo"),
            ("2023-01-09", "Día de los Santos Reyes"),
            ("2023-01-21", "Día de la Altagracia"),
            ("2023-01-30", "Día de Duarte"),
            ("2023-02-27", "Día de Independencia"),
            ("2023-04-07", "Viernes Santo"),
            ("2023-05-01", "Día del Trabajo"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-08-14", "Día de la Restauración"),
            ("2023-09-24", "Día de las Mercedes"),
            ("2023-11-06", "Día de la Constitución"),
            ("2023-12-25", "Día de Navidad"),
        )

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Año Nuevo"),
            ("2024-01-06", "Día de los Santos Reyes"),
            ("2024-01-21", "Día de la Altagracia"),
            ("2024-01-29", "Día de Duarte"),
            ("2024-02-27", "Día de Independencia"),
            ("2024-03-29", "Viernes Santo"),
            ("2024-04-29", "Día del Trabajo"),
            ("2024-05-30", "Corpus Christi"),
            ("2024-08-16", "Día de la Restauración"),
            ("2024-09-24", "Día de las Mercedes"),
            ("2024-11-04", "Día de la Constitución"),
            ("2024-12-25", "Día de Navidad"),
        )

    def test_movable(self):
        self.assertHoliday(
            "1998-01-05",
            "1998-01-26",
            "1998-05-04",
            "1998-11-09",
            "1999-01-04",
            "1999-01-25",
            "1999-05-01",
            "1999-11-06",
            "2001-08-20",
            "2012-08-16",
            "2016-08-16",
        )

        self.assertNoHoliday(
            "1998-01-06",
            "1998-05-01",
            "1998-11-06",
            "1999-01-26",
            "2001-08-16",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-01-10", "Día de los Santos Reyes"),
            ("2022-01-21", "Día de la Altagracia"),
            ("2022-01-24", "Día de Duarte"),
            ("2022-02-27", "Día de Independencia"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-02", "Día del Trabajo"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-08-15", "Día de la Restauración"),
            ("2022-09-24", "Día de las Mercedes"),
            ("2022-11-06", "Día de la Constitución"),
            ("2022-12-25", "Día de Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-10", "Epiphany"),
            ("2022-01-21", "Lady of Altagracia"),
            ("2022-01-24", "Juan Pablo Duarte Day"),
            ("2022-02-27", "Independence Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-05-02", "Labor Day"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-08-15", "Restoration Day"),
            ("2022-09-24", "Our Lady of Mercedes Day"),
            ("2022-11-06", "Constitution Day"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-10", "Богоявлення"),
            ("2022-01-21", "День Богоматері Альтаграсія"),
            ("2022-01-24", "День Дуарте"),
            ("2022-02-27", "День незалежності"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-05-02", "День праці"),
            ("2022-06-16", "Свято Тіла і Крові Христових"),
            ("2022-08-15", "День реставрації"),
            ("2022-09-24", "День Богоматері Милосердя"),
            ("2022-11-06", "День Конституції"),
            ("2022-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_TO_PREV_FRI, SUN_TO_NEXT_MON


class Micronesia(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Micronesia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Federated_States_of_Micronesia>
        * <https://web.archive.org/web/20250605035443/https://www.timeanddate.com/holidays/micronesia/>
        * [2014 FSM Code](https://web.archive.org/web/20241213112548/http://www.fsmlaw.org/fsm/code/PDF/FSMCA2014Tit01.pdf)
        * [Section 601, 602, 603](https://web.archive.org/web/20020731194201/http://fsmlaw.org/fsm/code/title01/t01ch06.htm)
        * [United Nations Day](https://web.archive.org/web/20241213120703/http://www.fsmlaw.org/fsm/code/PDF/CODE%201/PL%207-20.pdf)
        * [FSM Veterans of Foreign Wars Day](https://web.archive.org/web/20241213120631/http://www.fsmlaw.org/fsm/code/PDF/CODE%201/PL%2013-38.pdf)
        * [Micronesian Culture and Tradition Day](https://web.archive.org/web/20241213114748/http://fsmlaw.org/fsm/code/PDF/CODE%201/PL%2016-27.pdf)
        * [Presidents Day](https://web.archive.org/web/20241213120719/http://www.fsmlaw.org/fsm/code/PDF/CODE%201/PL%2021-209.pdf)

    Subdivisions Holidays References:
        * Chuuk:
            * [Chuuk State Code, T. 1, Chap. 3](https://web.archive.org/web/20250214151354/http://fsmlaw.org/chuuk/code/title01/T01_Ch03.htm)
            * [State Charter Day](https://web.archive.org/web/20241213113932/http://fsmlaw.org/chuuk/pdf/tsl_a/TSL%203-10%20EV.pdf)
            * [Chuuk State Constitution Day](https://web.archive.org/web/20241213113821/http://fsmlaw.org/chuuk/pdf/csl_a/CSL%20190-03%20EV.pdf)
        * Kosrae:
            * [Kosrae State Code, Section 2.501](https://web.archive.org/web/20250211055635/http://fsmlaw.org/kosrae/code/title02/t02c05.htm)
            * [Seventh Korsae State Legislature](https://web.archive.org/web/20241213115951/https://fsmlaw.org/kosrae/Law/pdf/123.pdf)
            * [Fourth Kosrae State Legislature](https://web.archive.org/web/20241213115949/https://fsmlaw.org/kosrae/Law/pdf/Kosrae%20State%20Law%201989-90.pdf)
            * [State Law 4-32](https://web.archive.org/web/20250211151328/https://www.kosraestatelegislature.com/4th-ksl-state-laws)
            * [State Law 7-76](https://web.archive.org/web/20250211151710/https://www.kosraestatelegislature.com/7th-ksl-state-laws)
            * [State Law 9-50](https://web.archive.org/web/20250430191523/https://www.kosraestatelegislature.com/9th-ksl-state-laws)
            * [State Law 13-79](https://web.archive.org/web/20241211232231/https://www.kosraestatelegislature.com/_files/ugd/f1e94d_cc244ef15bea4c74ba81cb1a4621c30f.pdf)
        * Pohnpei:
            * [State Code 2012](https://web.archive.org/web/20250209195119/https://fsmlaw.org/pohnpei/code/pdf/pohnpei%20state%202012%20code.pdf)
            * [2025](https://web.archive.org/web/20250124225740/https://pohnpeistate.gov.fm/fy-2025-holiday-schedule/)
            * [Women's Day](https://web.archive.org/web/20250403093235/https://pohnpeistate.gov.fm/2022/02/22/congratulations-pohnpei-womens-council-and-all-who-supported-this-important-undertaking/)
        * [Yap](https://web.archive.org/web/20250223222339/http://fsmlaw.org/yap/code/title01/T01_Ch08.htm)

        According to Section 602:

        > All holidays set forth in section 601 shall, if they occur on a Saturday,
        > be observed on the preceding Friday, and shall, if they occur on a Sunday,
        > be observed on the following Monday.

        According to Kosrae State Legislature State Laws:

        * 4-32: Government Holidays were amended.
        * 4-135: September 8 was declared a public holiday.
        * 7-27: Good Friday was declared a public holiday.
        * 7-76: Gospel Day was declared a public holiday.
        * 9-50: Thanksgiving Day was declared a public holiday.
        * 13-79: Kosrae Disability Day was declared a public holiday.

        According to Pohnpei State Code 2012 Section 7-101:

        * Pohnpei Constitution Day is a public holiday.
        * Liberation Day is a public holiday.
        * Pohnpei Cultural Day is a public holiday.
        * Good Friday is a public holiday.
    """

    country = "FM"
    default_language = "en_FM"
    # %s (observed).
    observed_label = tr("%s (observed)")
    # Federated States of Micronesia gained independence on November 3, 1986.
    start_year = 1987
    subdivisions = (
        "KSA",  # Kosrae.
        "PNI",  # Pohnpei.
        "TRK",  # Chuuk.
        "YAP",  # Yap.
    )
    subdivisions_aliases = {
        "Kosrae": "KSA",
        "Kusaie": "KSA",
        "Pohnpei": "PNI",
        "Ponape": "PNI",
        "Chuuk": "TRK",
        "Truk": "TRK",
        "Yap": "YAP",
    }
    supported_languages = ("en_FM", "en_US")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SAT_TO_PREV_FRI + SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        name = tr("New Year's Day")
        self._add_observed(self._add_new_years_day(name))
        self._add_observed(self._next_year_new_years_day, name=name, rule=SAT_TO_PREV_FRI)

        if self._year >= 2011:
            self._add_observed(
                # Micronesian Culture and Tradition Day.
                self._add_holiday_mar_31(tr("Micronesian Culture and Tradition Day"))
            )

        # Federated States of Micronesia Day.
        self._add_observed(self._add_holiday_may_10(tr("Federated States of Micronesia Day")))

        if self._year >= 1991:
            # United Nations Day.
            self._add_observed(self._add_united_nations_day(tr("United Nations Day")))

        # Independence Day.
        self._add_observed(self._add_holiday_nov_3(tr("Independence Day")))

        if self._year >= 2004:
            # FSM Veterans of Foreign Wars Day.
            self._add_observed(self._add_remembrance_day(tr("FSM Veterans of Foreign Wars Day")))

        if self._year >= 2021:
            # Presidents Day.
            self._add_observed(self._add_holiday_nov_23(tr("Presidents Day")))

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Christmas Day")))

    # Kosrae.
    def _populate_subdiv_ksa_public_holidays(self):
        if self._year >= 1991:
            # Kosrae State Constitution Day.
            self._add_observed(self._add_holiday_jan_11(tr("Kosrae State Constitution Day")))

        if self._year >= 2000:
            # Good Friday.
            self._add_good_friday(tr("Good Friday"))

            # Gospel Day.
            self._add_observed(self._add_holiday_aug_21(tr("Gospel Day")))

        if self._year >= 1991:
            # Kosrae Liberation Day.
            self._add_observed(self._add_holiday_sep_8(tr("Kosrae Liberation Day")))

        # Self Government Day.
        self._add_observed(self._add_holiday_nov_3(tr("Self Government Day")))

        if self._year >= 2011:
            # Thanksgiving Day.
            self._add_holiday_4th_thu_of_nov(tr("Thanksgiving Day"))

        if self._year >= 2024:
            # Kosrae Disability Day.
            self._add_observed(self._add_holiday_dec_3(tr("Kosrae Disability Day")))

    # Pohnpei.
    def _populate_subdiv_pni_public_holidays(self):
        if self._year >= 2022:
            # Women's Day.
            self._add_observed(self._add_womens_day(tr("Women's Day")))

        # Pohnpei Cultural Day.
        self._add_observed(self._add_holiday_mar_31(tr("Pohnpei Cultural Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Liberation Day.
        self._add_observed(self._add_holiday_sep_11(tr("Liberation Day")))

        # Pohnpei Constitution Day.
        self._add_observed(self._add_holiday_nov_8(tr("Pohnpei Constitution Day")))

    # Chuuk.
    def _populate_subdiv_trk_public_holidays(self):
        # State Charter Day.
        self._add_observed(self._add_holiday_sep_26(tr("State Charter Day")))

        if self._year >= 1990:
            # Chuuk State Constitution Day.
            self._add_observed(self._add_holiday_oct_1(tr("Chuuk State Constitution Day")))

    # Yap.
    def _populate_subdiv_yap_public_holidays(self):
        # Yap Day.
        self._add_observed(self._add_holiday_mar_1(tr("Yap Day")))

        # Yap State Constitution Day.
        self._add_observed(self._add_holiday_dec_24(tr("Yap State Constitution Day")))


class FM(Micronesia):
    pass


class FSM(Micronesia):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.saint_helena_ascension_and_tristan_da_cunha import (
    SaintHelenaAscensionAndTristanDaCunha,
)
from tests.common import CommonCountryTests


class TestSaintHelenaAscensionAndTristanDaCunha(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(SaintHelenaAscensionAndTristanDaCunha, with_subdiv_categories=True)

    def test_special_holidays(self):
        for dt, name in (
            ("2018-02-09", "Final Departure of R.M.S. St Helena"),
            ("2022-06-03", "Queen Elizabeth II's Platinum Jubilee"),
            ("2022-09-19", "Queen Elizabeth II's State Funeral"),
            ("2023-01-24", "The Duke of Edinburgh's Visit"),
            ("2023-05-08", "Coronation of His Majesty King Charles III"),
        ):
            self.assertHolidayName(name, dt)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertGovernmentHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoGovernmentNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_queens_birthday(self):
        name = "Queen's Birthday"
        self.assertHolidayName(
            name,
            "2018-06-18",
            "2019-06-17",
            "2020-06-08",
            "2021-06-14",
            "2022-06-13",
        )
        self.assertHolidayName(name, range(self.start_year, 2023))
        self.assertNoHolidayName(name, range(2023, self.end_year))

    def test_kings_birthday(self):
        name = "King's Birthday"
        self.assertHolidayName(
            name,
            "2023-06-19",
            "2024-11-15",
            "2025-11-14",
        )
        self.assertHolidayName(name, range(2023, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2023))

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertGovernmentHolidayName(name, self.full_range)

    def test_ascension_day(self):
        name = "Ascension Day"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"AC", "TA"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-05-21",
                    "2021-05-13",
                    "2022-05-26",
                    "2023-05-18",
                    "2024-05-09",
                    "2025-05-29",
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_helena_day(self):
        name = "Saint Helena Day"
        self.assertNoHolidayName(name)
        obs_dts = (
            "2016-05-20",
            "2017-05-22",
            "2022-05-20",
            "2023-05-22",
        )
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "HL":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-21" for year in self.full_range)
                )
                self.assertHolidayName(f"{name} (observed)", holidays, obs_dts)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], obs_dts)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_ratting_day(self):
        name = "Ratting Day"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "TA":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2015-05-16",
                    "2016-04-30",
                    "2017-05-26",
                    "2018-06-02",
                    "2019-05-24",
                    "2020-04-25",
                    "2021-04-09",
                    "2023-06-02",
                    "2025-05-30",
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_anniversary_day(self):
        name = "Anniversary Day"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "TA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-14" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_august_bank_holiday(self):
        name = "August Bank Holiday"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_government_holidays.items():
            if subdiv in {"AC", "HL"}:
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-08-31",
                    "2021-08-30",
                    "2022-08-29",
                    "2023-08-28",
                    "2024-08-26",
                    "2025-08-25",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-21", "Saint Helena Day"),
            ("2025-05-29", "Ascension Day"),
            ("2025-05-30", "Ratting Day"),
            ("2025-06-09", "Whit Monday"),
            ("2025-08-14", "Anniversary Day"),
            ("2025-08-25", "August Bank Holiday"),
            ("2025-11-14", "King's Birthday"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-21", "Saint Helena Day"),
            ("2025-05-29", "Ascension Day"),
            ("2025-05-30", "Ratting Day"),
            ("2025-06-09", "Whit Monday"),
            ("2025-08-14", "Anniversary Day"),
            ("2025-08-25", "August Bank Holiday"),
            ("2025-11-14", "King's Birthday"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from collections.abc import Mapping
from datetime import date
from pathlib import Path

CLASS_TEMPLATE = """class {class_name}:
{holiday_data}
"""

HOLIDAY_DATA_TEMPLATE = """    {hol_name}_DATES = {{
{year_dates}
    }}
"""

YEAR_TEMPLATE = "        {year}: {dates},"

MONTHS = ("JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC")


class CalendarGenerator:
    """Generate calendar data mappings with precomputed holiday dates."""

    class_name: str
    outfile: Path

    def __init__(self, calendar_name: str = "calendar", class_name: str = "Lunisolar") -> None:
        self.class_name = class_name
        self.outfile = (
            Path(__file__).parents[2] / "holidays" / "calendars" / f"{calendar_name}_dates.py"
        )

    def generate(self, dates: Mapping[str, Mapping[int, date | list[date]]]) -> None:
        holiday_data = []
        for hol_name, hol_dates in sorted(dates.items()):
            year_dates = []
            for year, dts in sorted(hol_dates.items()):
                dates_list = [dts] if isinstance(dts, date) else dts
                dates_str = ", ".join(f"({MONTHS[dt.month - 1]}, {dt.day})" for dt in dates_list)
                if len(dates_list) > 1:
                    dates_str = f"({dates_str})"
                year_dates.append(YEAR_TEMPLATE.format(year=year, dates=dates_str))
            holiday_data.append(
                HOLIDAY_DATA_TEMPLATE.format(hol_name=hol_name, year_dates="\n".join(year_dates))
            )
        self.outfile.write_text(
            CLASS_TEMPLATE.format(
                class_name=self.class_name, holiday_data="\n".join(holiday_data)
            ),
            encoding="utf-8",
            newline="\n",
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.financial.germany_exchange import GermanyStockExchange
from tests.common import CommonFinancialTests


class TestGermanyStockExchange(CommonFinancialTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(GermanyStockExchange)

    def test_special_holidays(self):
        self.assertHoliday("2017-10-31")

    def test_new_years_day(self):
        self.assertHolidayName("Neujahr", (f"{year}-01-01" for year in self.full_range))

    def test_good_friday(self):
        name = "Karfreitag"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Ostermontag"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Erster Mai", (f"{year}-05-01" for year in self.full_range))

    def test_whit_monday(self):
        name = "Pfingstmontag"
        self.assertHolidayName(
            name,
            "2016-05-16",
            "2017-06-05",
            "2018-05-21",
            "2019-06-10",
            "2020-06-01",
            "2021-05-24",
        )
        self.assertNoHolidayName(name, range(2022, self.end_year))

    def test_german_unity_day(self):
        name = "Tag der Deutschen Einheit"
        self.assertHolidayName(name, (f"{year}-10-03" for year in range(self.start_year, 2022)))
        self.assertNoHolidayName(name, range(2022, self.end_year))

    def test_christmas_eve(self):
        self.assertHolidayName("Heiligabend", (f"{year}-12-24" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName(
            "Erster Weihnachtstag", (f"{year}-12-25" for year in self.full_range)
        )

    def test_second_day_of_christmas(self):
        self.assertHolidayName(
            "Zweiter Weihnachtstag", (f"{year}-12-26" for year in self.full_range)
        )

    def test_new_years_eve(self):
        self.assertHolidayName("Silvester", (f"{year}-12-31" for year in self.full_range))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2020-01-01", "Neujahr"),
            ("2020-04-10", "Karfreitag"),
            ("2020-04-13", "Ostermontag"),
            ("2020-05-01", "Erster Mai"),
            ("2020-06-01", "Pfingstmontag"),
            ("2020-10-03", "Tag der Deutschen Einheit"),
            ("2020-12-24", "Heiligabend"),
            ("2020-12-25", "Erster Weihnachtstag"),
            ("2020-12-26", "Zweiter Weihnachtstag"),
            ("2020-12-31", "Silvester"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2020-01-01", "New Year's Day"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-13", "Easter Monday"),
            ("2020-05-01", "Labor Day"),
            ("2020-06-01", "Whit Monday"),
            ("2020-10-03", "German Unity Day"),
            ("2020-12-24", "Christmas Eve"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-26", "Second Day of Christmas"),
            ("2020-12-31", "New Year's Eve"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2020-01-01", "วันขึ้นปีใหม่"),
            ("2020-04-10", "วันศุกร์ประเสริฐ"),
            ("2020-04-13", "วันจันทร์อีสเตอร์"),
            ("2020-05-01", "วันแรงงาน"),
            ("2020-06-01", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2020-10-03", "วันรวมชาติเยอรมัน"),
            ("2020-12-24", "วันคริสต์มาสอีฟ"),
            ("2020-12-25", "วันคริสต์มาสวันแรก"),
            ("2020-12-26", "วันคริสต์มาสวันที่สอง"),
            ("2020-12-31", "วันสิ้นปี"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2020-01-01", "Новий рік"),
            ("2020-04-10", "Страсна пʼятниця"),
            ("2020-04-13", "Великодній понеділок"),
            ("2020-05-01", "День праці"),
            ("2020-06-01", "День Святого Духа"),
            ("2020-10-03", "День німецької єдності"),
            ("2020-12-24", "Святий вечір"),
            ("2020-12-25", "Перший день Різдва"),
            ("2020-12-26", "Другий день Різдва"),
            ("2020-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.nauru import Nauru
from tests.common import CommonCountryTests


class TestNauru(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1969, 2050)
        super().setUpClass(Nauru, years=years, years_non_observed=years)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1969, 2050)))
        dt = (
            "2000-01-03",
            "2005-01-03",
            "2006-01-02",
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_independence_day(self):
        name = "Independence Day"
        name_after = f"Day following {name}"
        self.assertHolidayName(name, (f"{year}-01-31" for year in range(1969, 2050)))
        self.assertHolidayName(name_after, (f"{year}-02-01" for year in range(1969, 2050)))
        dt = (
            "2004-02-02",
            "2009-02-02",
            "2010-02-02",
            "2015-02-02",
            "2016-02-02",
            "2021-02-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        dt_after = (
            "2003-02-03",
            "2004-02-03",
            "2009-02-03",
            "2014-02-03",
            "2015-02-03",
            "2020-02-03",
        )
        self.assertHolidayName(f"{name_after} (observed)", dt_after)
        self.assertNoNonObservedHoliday(dt, dt_after)

    def test_international_womens_day(self):
        name = "International Women's Day"
        self.assertHolidayName(name, (f"{year}-03-08" for year in range(2019, 2050)))
        self.assertNoHolidayName(name, range(1969, 2019))
        dt = (
            "2020-03-09",
            "2025-03-10",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(1969, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1969, 2050))

    def test_easter_tuesday(self):
        name = "Easter Tuesday"
        self.assertHolidayName(
            name,
            "2020-04-14",
            "2021-04-06",
            "2022-04-19",
            "2023-04-11",
            "2024-04-02",
            "2025-04-22",
        )
        self.assertHolidayName(name, range(1969, 2050))

    def test_constitution_day(self):
        name = "Constitution Day"
        self.assertHolidayName(name, (f"{year}-05-17" for year in range(1969, 2050)))
        dt = (
            "2003-05-19",
            "2008-05-19",
            "2009-05-18",
            "2014-05-19",
            "2015-05-18",
            "2020-05-18",
            "2025-05-19",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_ronphos_handover(self):
        name = "RONPHOS Handover"
        self.assertHolidayName(name, (f"{year}-07-01" for year in range(2018, 2050)))
        self.assertNoHolidayName(name, range(1969, 2018))
        dt = (
            "2018-07-02",
            "2023-07-03",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_ibumin_earoeni_day(self):
        name = "Ibumin Earoeni Day"
        self.assertHolidayName(name, (f"{year}-08-19" for year in range(2019, 2050)))
        self.assertNoHolidayName(name, range(1969, 2019))
        dt = ("2023-08-21",)
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_national_youth_day(self):
        name = "National Youth Day"
        self.assertHolidayName(name, (f"{year}-09-25" for year in range(2001, 2020)))
        self.assertNoHolidayName(name, range(1969, 2001), range(2020, 2050))
        dt = (
            "2004-09-27",
            "2005-09-26",
            "2010-09-27",
            "2011-09-26",
            "2016-09-26",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_sir_hammer_deroburt_day(self):
        name = "Sir Hammer DeRoburt Day"
        self.assertHolidayName(name, (f"{year}-09-25" for year in range(2020, 2050)))
        self.assertNoHolidayName(name, range(1969, 2020))
        dt = (
            "2021-09-27",
            "2022-09-26",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_angam_day(self):
        name = "Angam Day"
        self.assertHolidayName(name, (f"{year}-10-26" for year in range(1969, 2050)))
        dt = (
            "2002-10-28",
            "2003-10-27",
            "2008-10-27",
            "2013-10-28",
            "2014-10-27",
            "2019-10-28",
            "2024-10-28",
            "2025-10-27",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(1969, 2050)))
        dt = (
            "2004-12-27",
            "2005-12-27",
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_day_following_christmas(self):
        name = "Day following Christmas"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(1969, 2050)))
        dt = (
            "2004-12-28",
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-01-31", "Independence Day"),
            ("2024-02-01", "Day following Independence Day"),
            ("2024-03-08", "International Women's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-02", "Easter Tuesday"),
            ("2024-05-17", "Constitution Day"),
            ("2024-07-01", "RONPHOS Handover"),
            ("2024-08-19", "Ibumin Earoeni Day"),
            ("2024-09-25", "Sir Hammer DeRoburt Day"),
            ("2024-10-26", "Angam Day"),
            ("2024-10-28", "Angam Day (observed)"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Day following Christmas"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-01-31", "Independence Day"),
            ("2025-02-01", "Day following Independence Day"),
            ("2025-02-03", "Day following Independence Day (observed)"),
            ("2025-03-08", "International Women's Day"),
            ("2025-03-10", "International Women's Day (observed)"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-04-22", "Easter Tuesday"),
            ("2025-05-17", "Constitution Day"),
            ("2025-05-19", "Constitution Day (observed)"),
            ("2025-07-01", "RONPHOS Handover"),
            ("2025-08-19", "Ibumin Earoeni Day"),
            ("2025-09-25", "Sir Hammer DeRoburt Day"),
            ("2025-10-26", "Angam Day"),
            ("2025-10-27", "Angam Day (observed)"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Day following Christmas"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-31", "Independence Day"),
            ("2025-02-01", "Day following Independence Day"),
            ("2025-02-03", "Day following Independence Day (observed)"),
            ("2025-03-08", "International Women's Day"),
            ("2025-03-10", "International Women's Day (observed)"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-04-22", "Easter Tuesday"),
            ("2025-05-17", "Constitution Day"),
            ("2025-05-19", "Constitution Day (observed)"),
            ("2025-07-01", "RONPHOS Handover"),
            ("2025-08-19", "Ibumin Earoeni Day"),
            ("2025-09-25", "Sir Hammer DeRoburt Day"),
            ("2025-10-26", "Angam Day"),
            ("2025-10-27", "Angam Day (observed)"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Day following Christmas"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.united_states_minor_outlying_islands import (
    UnitedStatesMinorOutlyingIslands,
)
from tests.common import CommonCountryTests


class TestUnitedStatesMinorOutlyingIslands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(UnitedStatesMinorOutlyingIslands)

    def test_2024_public_holidays(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-01-15", "Martin Luther King Jr. Day"),
            ("2024-02-19", "Washington's Birthday"),
            ("2024-05-27", "Memorial Day"),
            ("2024-06-19", "Juneteenth National Independence Day"),
            ("2024-07-04", "Independence Day"),
            ("2024-09-02", "Labor Day"),
            ("2024-11-11", "Veterans Day"),
            ("2024-11-28", "Thanksgiving Day"),
            ("2024-12-25", "Christmas Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-17", "Martin Luther King Jr. Day"),
            ("2022-02-14", "Valentine's Day"),
            ("2022-02-21", "Washington's Birthday"),
            ("2022-03-17", "Saint Patrick's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-05-08", "Mother's Day"),
            ("2022-05-30", "Memorial Day"),
            ("2022-06-19", "Father's Day; Juneteenth National Independence Day"),
            ("2022-06-20", "Juneteenth National Independence Day (observed)"),
            ("2022-07-04", "Independence Day"),
            ("2022-09-05", "Labor Day"),
            ("2022-10-31", "Halloween"),
            ("2022-11-11", "Veterans Day"),
            ("2022-11-24", "Thanksgiving Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-17", "วันมาร์ติน ลูเทอร์ คิง จูเนียร์"),
            ("2022-02-14", "วันวาเลนไทน์"),
            ("2022-02-21", "วันเกิดวอชิงตัน"),
            ("2022-03-17", "วันนักบุญแพทริก"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-05-08", "วันแม่"),
            ("2022-05-30", "วันรำลึก"),
            ("2022-06-19", "วันประกาศอิสรภาพแห่งชาติจูนทีนท์; วันพ่อ"),
            ("2022-06-20", "ชดเชยวันประกาศอิสรภาพแห่งชาติจูนทีนท์"),
            ("2022-07-04", "วันประกาศอิสรภาพ"),
            ("2022-09-05", "วันแรงงาน"),
            ("2022-10-31", "วันฮาโลวีน"),
            ("2022-11-11", "วันทหารผ่านศึก"),
            ("2022-11-24", "วันขอบคุณพระเจ้า"),
            ("2022-12-24", "วันคริสต์มาสอีฟ"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "ชดเชยวันคริสต์มาส"),
            ("2022-12-31", "วันสิ้นปี"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.constants import OPTIONAL, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Greenland(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Greenland holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Greenland>
        * [Greenlandic names source](https://web.archive.org/web/20241214072442/https://nalunaarutit.gl/groenlandsk-lovgivning/2008/bkg-26-2008?sc_lang=kl-GL)
        * [Translation source](https://web.archive.org/web/20250422014548/https://www.norden.org/en/info-norden/public-holidays-greenland)
    """

    country = "GL"
    default_language = "kl"
    supported_categories = (OPTIONAL, PUBLIC)
    supported_languages = ("da", "en_US", "fi", "is", "kl", "no", "sv", "uk")
    # Greenland Home Rule Act 1978.
    start_year = 1979

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Ukiortaaq"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Sisamanngortoq illernartoq"))

        # Good Friday.
        self._add_good_friday(tr("Tallimanngorneq tannaartoq"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Poorskip ullua"))

        # Easter Monday.
        self._add_easter_monday(tr("Poorskip-aappaa"))

        # Great Prayer Day.
        self._add_holiday_26_days_past_easter(tr("Ulloq qinuffiusoq"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Qilaliarfik"))

        # Whit Sunday.
        self._add_whit_sunday(tr("Piinsip ullua"))

        # Whit Monday.
        self._add_whit_monday(tr("Piinsip-aappaa"))

        # Christmas Day.
        self._add_christmas_day(tr("Juullip ullua"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Juullip-aappaa"))

    def _populate_optional_holidays(self):
        # Epiphany.
        self._add_epiphany_day(tr("Kunngit pingasut ulluat"))

        # International Workers' Day.
        self._add_labor_day(tr("Sulisartut ulluat"))

        if self._year >= 1983:
            # National Day.
            self._add_holiday_jun_21(tr("Ullortuneq"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Juulliaraq"))

        # New Year's Eve.
        self._add_new_years_eve(tr("Ukiortaami"))


class GL(Greenland):
    pass


class GRL(Greenland):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.grenada import Grenada
from tests.common import CommonCountryTests


class TestGrenada(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Grenada)

    def test_special_holidays(self):
        self.assertHoliday(
            "2011-12-27",
            "2013-02-22",
            "2016-12-27",
            "2018-03-14",
            "2022-06-24",
            "2022-12-27",
            "2023-07-04",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-02-07" for year in self.full_range))
        obs_dts = (
            "2010-02-08",
            "2016-02-08",
            "2021-02-08",
            "2027-02-08",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_corpus_christi(self):
        name = "Corpus Christi"
        self.assertHolidayName(
            name,
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertHolidayName(
            name,
            "2020-08-03",
            "2021-08-02",
            "2022-08-01",
            "2023-08-07",
            "2024-08-05",
        )
        self.assertHolidayName(name, range(self.start_year, 2025))
        self.assertHolidayName(name, (f"{year}-08-01" for year in range(2025, self.end_year)))
        obs_dts = (
            "2027-08-02",
            "2032-08-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_carnival_monday(self):
        name = "Carnival Monday"
        self.assertHolidayName(
            name,
            "2020-08-10",
            "2021-08-09",
            "2022-08-08",
            "2023-08-14",
            "2024-08-12",
            "2025-08-11",
        )
        self.assertHolidayName(name, self.full_range)

    def test_carnival_tuesday(self):
        name = "Carnival Tuesday"
        self.assertHolidayName(
            name,
            "2020-08-11",
            "2021-08-10",
            "2022-08-09",
            "2023-08-15",
            "2024-08-13",
            "2025-08-12",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_heroes_day(self):
        name = "National Heroes' Day"
        self.assertHolidayName(name, (f"{year}-10-19" for year in range(2023, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2023))
        obs_dts = (
            "2025-10-20",
            "2031-10-20",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_thanksgiving_day(self):
        name = "Thanksgiving Day"
        self.assertHolidayName(name, (f"{year}-10-25" for year in self.full_range))
        obs_dts = (
            "2009-10-26",
            "2015-10-26",
            "2020-10-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2005-12-26",
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2012(self):
        # https://web.archive.org/web/20120623100105/http://www.gov.gd/holiday_events.html
        self.assertHolidaysInYear(
            2012,
            ("2012-01-01", "New Year's Day"),
            ("2012-01-02", "New Year's Day (observed)"),
            ("2012-02-07", "Independence Day"),
            ("2012-04-06", "Good Friday"),
            ("2012-04-09", "Easter Monday"),
            ("2012-05-01", "Labour Day"),
            ("2012-05-28", "Whit Monday"),
            ("2012-06-07", "Corpus Christi"),
            ("2012-08-06", "Emancipation Day"),
            ("2012-08-13", "Carnival Monday"),
            ("2012-08-14", "Carnival Tuesday"),
            ("2012-10-25", "Thanksgiving Day"),
            ("2012-12-25", "Christmas Day"),
            ("2012-12-26", "Boxing Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "New Year's Day"),
            ("2025-02-07", "Independence Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labour Day"),
            ("2025-06-09", "Whit Monday"),
            ("2025-06-19", "Corpus Christi"),
            ("2025-08-01", "Emancipation Day"),
            ("2025-08-11", "Carnival Monday"),
            ("2025-08-12", "Carnival Tuesday"),
            ("2025-10-19", "National Heroes' Day"),
            ("2025-10-20", "National Heroes' Day (observed)"),
            ("2025-10-25", "Thanksgiving Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-02-07", "Independence Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-06-09", "Whit Monday"),
            ("2025-06-19", "Corpus Christi"),
            ("2025-08-01", "Emancipation Day"),
            ("2025-08-11", "Carnival Monday"),
            ("2025-08-12", "Carnival Tuesday"),
            ("2025-10-19", "National Heroes' Day"),
            ("2025-10-20", "National Heroes' Day (observed)"),
            ("2025-10-25", "Thanksgiving Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.barbados import Barbados
from tests.common import CommonCountryTests


class TestBarbados(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Barbados, years_non_observed=range(2000, 2024))

    def test_special_holidays(self):
        self.assertHoliday(
            "2021-01-04",
            "2021-01-05",
            "2023-07-31",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_errol_barrow_day(self):
        name = "Errol Barrow Day"
        self.assertHolidayName(name, (f"{year}-01-21" for year in range(1989, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1989))
        obs_dts = (
            "2001-01-22",
            "2007-01-22",
            "2018-01-22",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_heroes_day(self):
        name = "National Heroes Day"
        self.assertHolidayName(name, (f"{year}-04-28" for year in range(1998, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1998))
        obs_dts = (
            "2002-04-29",
            "2013-04-29",
            "2019-04-29",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_may_day(self):
        name = "May Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "2018-05-21",
            "2019-06-10",
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertHolidayName(name, (f"{year}-08-01" for year in self.full_range))
        obs_dts = (
            "2004-08-03",
            "2005-08-02",
            "2010-08-03",
            "2011-08-02",
            "2016-08-02",
            "2021-08-03",
            "2022-08-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_kadooment_day(self):
        name = "Kadooment Day"
        self.assertHolidayName(
            name,
            "2018-08-06",
            "2019-08-05",
            "2020-08-03",
            "2021-08-02",
            "2022-08-01",
            "2023-08-07",
        )
        self.assertHolidayName(name, self.full_range)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-11-30" for year in self.full_range))
        obs_dts = (
            "2003-12-01",
            "2008-12-01",
            "2014-12-01",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2005-12-27",
            "2011-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-21", "Errol Barrow Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-28", "National Heroes Day"),
            ("2022-05-01", "May Day"),
            ("2022-05-02", "May Day (observed)"),
            ("2022-06-06", "Whit Monday"),
            ("2022-08-01", "Emancipation Day; Kadooment Day"),
            ("2022-08-02", "Emancipation Day (observed)"),
            ("2022-11-30", "Independence Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-01-21", "Errol Barrow Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-04-28", "National Heroes Day"),
            ("2023-05-01", "May Day"),
            ("2023-05-29", "Whit Monday"),
            ("2023-07-31", "50th Anniversary of CARICOM Holiday"),
            ("2023-08-01", "Emancipation Day"),
            ("2023-08-07", "Kadooment Day"),
            ("2023-11-30", "Independence Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.dr_congo import DRCongo
from tests.common import CommonCountryTests


class TestDRCongo(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(DRCongo)

    def test_new_years_day(self):
        name = "Nouvel an"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))

        obs_dts = (
            "2005-12-31",
            "2011-12-31",
            "2016-12-31",
            "2022-12-31",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_martyrs_day(self):
        name = "Martyrs de l'indépendance"
        self.assertHolidayName(name, (f"{year}-01-04" for year in range(2015, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2015))

        obs_dts = ("2015-01-03",)
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_hero_laurent_desire_kabila_day(self):
        name = "Journée du héros national Laurent Désiré Kabila"
        self.assertHolidayName(name, (f"{year}-01-16" for year in range(2015, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2015))

        obs_dts = ("2022-01-15",)
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_hero_patrice_emery_lumumba_day(self):
        name = "Journée du héros national Patrice Emery Lumumba"
        self.assertHolidayName(name, (f"{year}-01-17" for year in range(2015, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2015))

        obs_dts = (
            "2016-01-16",
            "2021-01-16",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)

    def test_day_of_the_struggle_of_simon_kimbangu_and_african_consciousness(self):
        name = "Journée du combat de Simon Kimbangu et de la conscience africaine"
        self.assertHolidayName(name, (f"{year}-04-06" for year in range(2023, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2023))

        obs_dts = ("2025-04-05",)
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_armed_forces_day(self):
        name_1980 = "Fête des Forces armées zaïroises"
        name_2014 = "Journée de la Révolution et des Forces Armées"

        self.assertHolidayName(
            name_1980, (f"{year}-11-17" for year in range(self.start_year, 2014))
        )
        self.assertHolidayName(name_2014, (f"{year}-05-17" for year in range(2014, self.end_year)))
        self.assertNoHolidayName(name_1980, range(2014, self.end_year))
        self.assertNoHolidayName(name_2014, range(self.start_year, 2014))

        obs_dts_1980 = (
            "2002-11-16",
            "2013-11-16",
        )
        obs_dts_2014 = (
            "2015-05-16",
            "2020-05-16",
        )
        self.assertHolidayName(f"{name_1980} (observé)", obs_dts_1980)
        self.assertHolidayName(f"{name_2014} (observé)", obs_dts_2014)
        self.assertNoNonObservedHoliday(obs_dts_1980, obs_dts_2014)

    def test_labor_day(self):
        name = "Fête du travail"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))

        obs_dts = (
            "2005-04-30",
            "2011-04-30",
            "2016-04-30",
            "2022-04-30",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_anniversary_of_the_popular_movement_of_the_revolution(self):
        name = "Anniversaire du Mouvement populaire de la révolution"
        self.assertHolidayName(name, (f"{year}-05-20" for year in range(self.start_year, 2014)))
        self.assertNoHolidayName(name, range(2014, self.end_year))

        obs_dts = (
            "2001-05-19",
            "2007-05-19",
            "2012-05-19",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_anniversary_of_the_new_revolutionary_constitution(self):
        name = "Anniversaire de la nouvelle Constitution révolutionnaire"
        self.assertHolidayName(name, (f"{year}-06-24" for year in range(self.start_year, 2014)))
        self.assertNoHolidayName(name, range(2014, self.end_year))

        obs_dts = (
            "2001-06-23",
            "2007-06-23",
            "2012-06-23",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Journée de l'indépendance"
        self.assertHolidayName(name, (f"{year}-06-30" for year in self.full_range))

        obs_dts = (
            "2002-06-29",
            "2013-06-29",
            "2019-06-29",
            "2024-06-29",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_parents_day(self):
        name = "Fête des parents"
        self.assertHolidayName(name, (f"{year}-08-01" for year in self.full_range))

        obs_dts = (
            "2004-07-31",
            "2010-07-31",
            "2021-07-31",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_congolese_genocide_memorial_day(self):
        name = "Journée commémorative du génocide Congolais"
        self.assertHolidayName(name, (f"{year}-08-02" for year in range(2024, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2024))

        obs_dts = ("2026-08-01",)
        self.assertHolidayName(f"{name} (observé)", obs_dts)

    def test_youth_day(self):
        name = "Journée de la Jeunesse"
        self.assertHolidayName(name, (f"{year}-10-14" for year in range(self.start_year, 2014)))
        self.assertNoHolidayName(name, range(2014, self.end_year))

        obs_dts = (
            "2001-10-13",
            "2007-10-13",
            "2012-10-13",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_anniversary_of_the_new_regime(self):
        name = "Anniversaire du nouveau régime"
        self.assertHolidayName(name, (f"{year}-11-24" for year in range(self.start_year, 2014)))
        self.assertNoHolidayName(name, range(2014, self.end_year))

        obs_dts = (
            "2002-11-23",
            "2013-11-23",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Noël"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))

        obs_dts = (
            "2005-12-24",
            "2011-12-24",
            "2016-12-24",
            "2022-12-24",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Nouvel an"),
            ("2024-01-04", "Martyrs de l'indépendance"),
            ("2024-01-16", "Journée du héros national Laurent Désiré Kabila"),
            ("2024-01-17", "Journée du héros national Patrice Emery Lumumba"),
            ("2024-04-06", "Journée du combat de Simon Kimbangu et de la conscience africaine"),
            ("2024-05-01", "Fête du travail"),
            ("2024-05-17", "Journée de la Révolution et des Forces Armées"),
            ("2024-06-29", "Journée de l'indépendance (observé)"),
            ("2024-06-30", "Journée de l'indépendance"),
            ("2024-08-01", "Fête des parents"),
            ("2024-08-02", "Journée commémorative du génocide Congolais"),
            ("2024-12-25", "Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-04", "Martyrs' Day"),
            ("2024-01-16", "National Hero Laurent Désiré Kabila Day"),
            ("2024-01-17", "National Hero Patrice Emery Lumumba Day"),
            ("2024-04-06", "Day of the Struggle of Simon Kimbangu and African Consciousness"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-17", "Revolution and Armed Forces Day"),
            ("2024-06-29", "Independence Day (observed)"),
            ("2024-06-30", "Independence Day"),
            ("2024-08-01", "Parents' Day"),
            ("2024-08-02", "Congolese Genocide Memorial Day"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import unittest

from holidays.calendars.julian import julian_calendar_drift


class TestJulianCalendar(unittest.TestCase):
    def test_julian_calendar_drift(self):
        known_julian_calendar_drift = {
            1400: -13,
            1500: -13,
            1581: -13,
            1582: -13,
            1583: -3,
            1600: -3,
            1700: -2,
            1800: -1,
            1900: 0,
            2000: 0,
            2025: 0,
            2100: +1,
            2200: +2,
        }
        for year in known_julian_calendar_drift:
            self.assertEqual(known_julian_calendar_drift[year], julian_calendar_drift(year))



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.maldives import Maldives
from tests.common import CommonCountryTests


class TestMaldives(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Maldives, years=range(1950, 2050))

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in range(1950, 2050)))

    def test_labor_day(self):
        self.assertHolidayName("Labor Day", (f"{year}-05-01" for year in range(1950, 2050)))

    def test_independence_day(self):
        self.assertHolidayName("Independence Day", (f"{year}-07-26" for year in range(1950, 2050)))

    def test_victory_day(self):
        self.assertHolidayName("Victory Day", (f"{year}-11-03" for year in range(1950, 2050)))

    def test_republic_day(self):
        self.assertHolidayName("Republic Day", (f"{year}-11-11" for year in range(1950, 2050)))

    def test_2018(self):
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "New Year's Day"),
            ("2018-05-01", "Labor Day"),
            ("2018-05-16", "Beginning of Ramadan (estimated)"),
            ("2018-06-15", "Eid al-Fitr (estimated)"),
            ("2018-06-16", "Eid al-Fitr (estimated)"),
            ("2018-06-17", "Eid al-Fitr (estimated)"),
            ("2018-07-26", "Independence Day"),
            ("2018-08-20", "Hajj Day (estimated)"),
            ("2018-08-21", "Eid al-Adha (estimated)"),
            ("2018-08-22", "Eid al-Adha (estimated)"),
            ("2018-08-23", "Eid al-Adha (estimated)"),
            ("2018-08-24", "Eid al-Adha (estimated)"),
            ("2018-09-11", "Islamic New Year (estimated)"),
            ("2018-11-03", "Victory Day"),
            ("2018-11-09", "National Day (estimated)"),
            ("2018-11-11", "Republic Day"),
            ("2018-11-20", "Mawlid al-Nabi (estimated)"),
            ("2018-12-08", "The Day Maldives Embraced Islam (estimated)"),
        )

    def test_2020(self):
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-04-24", "Beginning of Ramadan (estimated)"),
            ("2020-05-01", "Labor Day"),
            ("2020-05-24", "Eid al-Fitr (estimated)"),
            ("2020-05-25", "Eid al-Fitr (estimated)"),
            ("2020-05-26", "Eid al-Fitr (estimated)"),
            ("2020-07-26", "Independence Day"),
            ("2020-07-30", "Hajj Day (estimated)"),
            ("2020-07-31", "Eid al-Adha (estimated)"),
            ("2020-08-01", "Eid al-Adha (estimated)"),
            ("2020-08-02", "Eid al-Adha (estimated)"),
            ("2020-08-03", "Eid al-Adha (estimated)"),
            ("2020-08-20", "Islamic New Year (estimated)"),
            ("2020-10-18", "National Day (estimated)"),
            ("2020-10-29", "Mawlid al-Nabi (estimated)"),
            ("2020-11-03", "Victory Day"),
            ("2020-11-11", "Republic Day"),
            ("2020-11-16", "The Day Maldives Embraced Islam (estimated)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)


from unittest import TestCase

from holidays.countries.rwanda import Rwanda
from tests.common import CommonCountryTests


class TestRwanda(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(2012, 2050)
        super().setUpClass(Rwanda, years=years, years_non_observed=years)
        cls.no_estimated_holidays = Rwanda(years=years, islamic_show_estimated=False)

    def test_new_years_day(self):
        name = "Ubunani"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(2012, 2050)))
        dt = (
            "2022-01-03",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (yizihijwe)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_day_after_new_years_day(self):
        name = "Umunsi ukurikira Ubunani"
        self.assertHolidayName(name, (f"{year}-01-02" for year in range(2016, 2050)))
        self.assertNoHolidayName(name, range(2012, 2016))
        dt = (
            "2021-01-04",
            "2027-01-04",
        )
        self.assertHolidayName(f"{name} (yizihijwe)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_national_heroes_day(self):
        name = "Umunsi w'Intwari"
        self.assertHolidayName(name, (f"{year}-02-01" for year in range(2012, 2050)))
        dt = (
            "2020-02-03",
            "2025-02-03",
        )
        self.assertHolidayName(f"{name} (yizihijwe)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_genocide_memorial_day(self):
        self.assertHolidayName(
            "Umunsi wo Kwibuka Jenoside yakorewe Abatutsi mu 1994",
            (f"{year}-04-07" for year in range(2012, 2050)),
        )

    def test_good_friday(self):
        name = "Umunsi wa Gatanu Mutagatifu"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(2012, 2050))

    def test_easter_monday(self):
        name = "Ku wa mbere wa Pasika"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(2017, 2050))
        self.assertNoHolidayName(name, range(2012, 2017))

    def test_labor_day(self):
        name = "Umunsi Mukuru w'Umurimo"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(2012, 2050)))
        dt = (
            "2021-05-03",
            "2022-05-03",
        )
        self.assertHolidayName(f"{name} (yizihijwe)", dt)

    def test_independence_day(self):
        name = "Umunsi w'Ubwigenge"
        self.assertHolidayName(name, (f"{year}-07-01" for year in range(2012, 2050)))
        dt = (
            "2017-07-03",
            "2018-07-02",
            "2023-07-03",
        )
        self.assertHolidayName(f"{name} (yizihijwe)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_liberation_day(self):
        name = "Umunsi wo Kwibohora"
        self.assertHolidayName(name, (f"{year}-07-04" for year in range(2012, 2050)))
        dt = (
            "2020-07-06",
            "2021-07-05",
        )
        self.assertHolidayName(f"{name} (yizihijwe)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_umuganura_day(self):
        name = "Umunsi w'Umuganura"
        self.assertHolidayName(
            name,
            "2020-08-07",
            "2021-08-06",
            "2022-08-05",
            "2023-08-04",
            "2024-08-02",
            "2025-08-01",
        )
        self.assertHolidayName(name, range(2015, 2050))
        self.assertNoHolidayName(name, range(2012, 2015))

    def test_assumption_day(self):
        name = "Ijyanwa mu Ijuru rya Bikiramariya"
        self.assertHolidayName(name, (f"{year}-08-15" for year in range(2012, 2050)))
        dt = (
            "2020-08-17",
            "2021-08-16",
        )
        self.assertHolidayName(f"{name} (yizihijwe)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_christmas_day(self):
        name = "Noheli"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(2012, 2050)))
        dt = (
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (yizihijwe)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_boxing_day(self):
        name = "Umunsi ukurikira Noheli"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(2012, 2050)))
        dt = (
            "2020-12-28",
            "2026-12-28",
        )
        self.assertHolidayName(f"{name} (yizihijwe)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_eid_al_fitr(self):
        name = "Eid El Fitr"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2012, 2050))

    def test_eid_al_adha(self):
        name = "Eid al-Adha"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-11",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2015, 2050))
        self.assertNoHolidayName(name, self.no_estimated_holidays, range(2012, 2015))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Ubunani"),
            ("2024-01-02", "Umunsi ukurikira Ubunani"),
            ("2024-02-01", "Umunsi w'Intwari"),
            ("2024-03-29", "Umunsi wa Gatanu Mutagatifu"),
            ("2024-04-01", "Ku wa mbere wa Pasika"),
            ("2024-04-07", "Umunsi wo Kwibuka Jenoside yakorewe Abatutsi mu 1994"),
            ("2024-04-10", "Eid El Fitr"),
            ("2024-05-01", "Umunsi Mukuru w'Umurimo"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Eid al-Adha (yizihijwe)"),
            ("2024-07-01", "Umunsi w'Ubwigenge"),
            ("2024-07-04", "Umunsi wo Kwibohora"),
            ("2024-08-02", "Umunsi w'Umuganura"),
            ("2024-08-15", "Ijyanwa mu Ijuru rya Bikiramariya"),
            ("2024-12-25", "Noheli"),
            ("2024-12-26", "Umunsi ukurikira Noheli"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-02", "Day after New Year's Day"),
            ("2024-02-01", "National Heroes' Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-07", "Memorial Day of Genocide perpetrated against the Tutsi in 1994"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-05-01", "Labor Day"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Eid al-Adha (observed)"),
            ("2024-07-01", "Independence Day"),
            ("2024-07-04", "Liberation Day"),
            ("2024-08-02", "Umuganura Day"),
            ("2024-08-15", "Assumption Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2024-01-01", "Nouvel An"),
            ("2024-01-02", "Le lendemain du Nouvel An"),
            ("2024-02-01", "Journée Nationale des Héros"),
            ("2024-03-29", "Vendredi Saint"),
            ("2024-04-01", "Lundi de Pâques"),
            ("2024-04-07", "Journée commémorative du Génocide perpétré contre les Tutsi en 1994"),
            ("2024-04-10", "Aïd el-Fitr"),
            ("2024-05-01", "Journée du Travail"),
            ("2024-06-16", "Aïd al-Adha"),
            ("2024-06-17", "Aïd al-Adha (observé)"),
            ("2024-07-01", "Journée de l'Indépendance"),
            ("2024-07-04", "Journée de la Libération"),
            ("2024-08-02", "Journée d'Umuganura"),
            ("2024-08-15", "Assomption"),
            ("2024-12-25", "Noël"),
            ("2024-12-26", "Le lendemain de Noël"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import BANK, GOVERNMENT
from holidays.countries.lebanon import Lebanon
from tests.common import CommonCountryTests


class TestLebanon(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Lebanon)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(
            Lebanon(categories=(BANK, GOVERNMENT), years=range(self.start_year, 2019))
        )

    def test_new_years_day(self):
        self.assertHolidayName(
            "رأس السنة الميلادية", (f"{year}-01-01" for year in self.full_range)
        )

    def test_armenian_orthodox_christmas(self):
        name = "عيد الميلاد عند الطوائف الارمنية الارثوذكسية"
        self.assertHolidayName(name, (f"{year}-01-06" for year in range(1995, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1995))

    def test_saint_maron_day(self):
        self.assertHolidayName("عيد مار مارون", (f"{year}-02-09" for year in self.full_range))

    def test_rafiki_memorial_day(self):
        name = "يوم ذكرى رفيق الحريري"
        self.assertNoHolidayName(name)
        self.assertBankHolidayName(name, (f"{year}-02-14" for year in range(2020, self.end_year)))
        self.assertGovernmentHolidayName(
            name, (f"{year}-02-14" for year in range(2020, self.end_year))
        )
        self.assertNoBankHolidayName(name, range(self.start_year, 2020))
        self.assertNoGovernmentHolidayName(name, range(self.start_year, 2020))

    def test_feast_of_the_annunciation(self):
        name = "عيد بشارة السيدة مريم العذراء"
        self.assertHolidayName(
            name, (f"{year}-03-25" for year in (*range(1995, 2006), *range(2010, self.end_year)))
        )
        self.assertNoHolidayName(name, range(self.start_year, 1995), range(2006, 2010))

    def test_catholic_good_friday(self):
        name = "الجمعة العظيمة عند الطوائف الكاثوليكية"
        self.assertHolidayName(
            name,
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_orthodox_good_friday(self):
        name = "الجمعة العظيمة عند الطوائف الأرثوذكسية"
        self.assertHolidayName(
            name,
            "2019-04-26",
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_orthodox_holy_saturday(self):
        name = "سبت النور للطائفة الأرثوذكسية"
        self.assertHolidayName(
            name,
            "2010-04-03",
            "2011-04-23",
            "2014-04-19",
            "2017-04-15",
        )
        self.assertNoHolidayName(name, set(range(2010, 2025)) - {2010, 2011, 2014, 2017})

    def test_catholic_easter_monday(self):
        name = "اثنين الفصح عند الطوائف الكاثوليكية"
        self.assertHolidayName(
            name,
            "1986-03-31",
            "1987-04-20",
            "1988-04-04",
            "1989-03-27",
            "1990-04-16",
        )
        self.assertHolidayName(name, range(1986, 1995))
        self.assertBankHolidayName(name, range(2020, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1986), range(1995, self.end_year))
        self.assertNoBankHolidayName(name, range(self.start_year, 2020))

    def test_orthodox_easter_monday(self):
        name = "اثنين الفصح عند الطوائف الأرثوذكسية"
        self.assertHolidayName(
            name,
            "1986-05-05",
            "1987-04-20",
            "1988-04-11",
            "1989-05-01",
            "1990-04-16",
        )
        self.assertHolidayName(name, range(1986, 1995))
        self.assertBankHolidayName(name, range(2020, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1986), range(1995, self.end_year))
        self.assertNoBankHolidayName(name, range(self.start_year, 2020))

    def test_orthodox_easter_tuesday(self):
        name = "ثلاثاء الفصح للطوائف الأرثوذكسية"
        self.assertHolidayName(
            name,
            "1987-04-21",
            "1990-04-17",
        )
        self.assertNoHolidayName(name, set(self.full_range) - {1987, 1990})

    def test_labor_day(self):
        name = "عيد العمل"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (يوم تعويضي)", obs_dts)

    def test_martyrs_day(self):
        name = "عيد الشهداء"
        self.assertHolidayName(
            name,
            "1978-05-07",
            "1979-05-06",
            "1992-05-10",
            "1993-05-09",
            "2006-05-07",
            "2007-05-06",
            "2008-05-04",
            "2009-05-03",
        )
        self.assertHolidayName(name, (*range(self.start_year, 1994), *range(2006, self.end_year)))
        self.assertNoHolidayName(name, range(1994, 2006))

    def test_resistance_and_liberation_day(self):
        name = "عيد المقاومة والتحرير"
        self.assertHolidayName(
            name,
            "2021-05-09",
            "2022-05-08",
            "2023-05-14",
            "2024-05-12",
            "2025-05-11",
        )
        self.assertHolidayName(name, range(2006, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2006))

    def test_anniversary_of_the_tragedy_of_beirut_port_explosion(self):
        name = "ذكرى مأساة انفجار مرفأ بيروت"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(
            name, (f"{year}-08-04" for year in range(2021, self.end_year))
        )
        self.assertNoGovernmentHolidayName(name, range(self.start_year, 2021))
        self.assertNoBankHolidayName(name)

    def test_assumption_day(self):
        self.assertHolidayName("عيد انتقال العذراء", (f"{year}-08-15" for year in self.full_range))

    def test_all_saints_days(self):
        name = "عيد جميع القديسين"
        self.assertHolidayName(name, (f"{year}-11-01" for year in range(1985, 1994)))
        self.assertNoHolidayName(name, range(self.start_year, 1985), range(1994, self.end_year))

    def test_independence_day(self):
        self.assertHolidayName("ذكرى الاستقلال", (f"{year}-11-22" for year in self.full_range))

    def test_christmas_day(self):
        self.assertHolidayName("عيد الميلاد", (f"{year}-12-25" for year in self.full_range))

    def test_islamic_new_year(self):
        name = "عيد رأس السنة الهجرية"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-20",
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_ashura(self):
        name = "عاشوراء"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-30",
            "2021-08-19",
            "2022-08-09",
            "2023-07-28",
            "2024-07-16",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_mawlid(self):
        name = "ذكرى المولد النبوي الشريف"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "1985-06-20",
            "1986-06-10",
            "1986-06-11",
            "1994-03-14",
            "1994-03-15",
            "1994-03-16",
            "2020-05-24",
            "2020-05-25",
            "2021-05-13",
            "2021-05-14",
            "2022-05-02",
            "2022-05-03",
            "2023-04-21",
            "2023-04-22",
            "2024-04-10",
            "2024-04-11",
            "2025-03-30",
            "2025-03-31",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        self.assertIslamicNoEstimatedHolidayNameCount(name, 1, range(self.start_year, 1986))
        self.assertIslamicNoEstimatedHolidayNameCount(name, 3, range(1986, 1995))
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 2, set(range(1995, self.end_year)) - {2000, 2033}
        )

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        self.assertHolidayName(
            name,
            "1985-08-27",
            "1985-08-28",
            "1986-08-16",
            "1986-08-17",
            "1986-08-18",
            "1994-05-21",
            "1994-05-22",
            "2020-07-31",
            "2020-08-01",
            "2021-07-20",
            "2021-07-21",
            "2022-07-09",
            "2022-07-10",
            "2023-06-28",
            "2023-06-29",
            "2024-06-16",
            "2024-06-17",
            "2025-06-06",
            "2025-06-07",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        self.assertIslamicNoEstimatedHolidayNameCount(
            name,
            2,
            range(self.start_year, 1986),
            set(range(1994, self.end_year)) - {2006, 2007, 2039},
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 3, range(1986, 1994))

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "رأس السنة الميلادية"),
            ("2024-01-06", "عيد الميلاد عند الطوائف الارمنية الارثوذكسية"),
            ("2024-02-09", "عيد مار مارون"),
            ("2024-03-25", "عيد بشارة السيدة مريم العذراء"),
            ("2024-03-29", "الجمعة العظيمة عند الطوائف الكاثوليكية"),
            ("2024-04-10", "عيد الفطر"),
            ("2024-04-11", "عيد الفطر"),
            ("2024-05-01", "عيد العمل"),
            ("2024-05-03", "الجمعة العظيمة عند الطوائف الأرثوذكسية"),
            ("2024-05-05", "عيد الشهداء"),
            ("2024-05-12", "عيد المقاومة والتحرير"),
            ("2024-06-16", "عيد الأضحى"),
            ("2024-06-17", "عيد الأضحى"),
            ("2024-07-07", "عيد رأس السنة الهجرية"),
            ("2024-07-16", "عاشوراء"),
            ("2024-08-15", "عيد انتقال العذراء"),
            ("2024-09-15", "ذكرى المولد النبوي الشريف"),
            ("2024-11-22", "ذكرى الاستقلال"),
            ("2024-12-25", "عيد الميلاد"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "رأس السنة الميلادية"),
            ("2025-01-06", "عيد الميلاد عند الطوائف الارمنية الارثوذكسية"),
            ("2025-02-09", "عيد مار مارون"),
            ("2025-02-14", "يوم ذكرى رفيق الحريري"),
            ("2025-03-25", "عيد بشارة السيدة مريم العذراء"),
            ("2025-03-30", "عيد الفطر"),
            ("2025-03-31", "عيد الفطر"),
            (
                "2025-04-18",
                "الجمعة العظيمة عند الطوائف الأرثوذكسية; الجمعة العظيمة عند الطوائف الكاثوليكية",
            ),
            ("2025-04-19", "سبت النور للطائفة الأرثوذكسية"),
            (
                "2025-04-21",
                "اثنين الفصح عند الطوائف الأرثوذكسية; اثنين الفصح عند الطوائف الكاثوليكية",
            ),
            ("2025-05-01", "عيد العمل"),
            ("2025-05-04", "عيد الشهداء"),
            ("2025-05-11", "عيد المقاومة والتحرير"),
            ("2025-06-06", "عيد الأضحى"),
            ("2025-06-07", "عيد الأضحى"),
            ("2025-06-26", "عيد رأس السنة الهجرية"),
            ("2025-07-05", "عاشوراء (تقديري)"),
            ("2025-08-04", "ذكرى مأساة انفجار مرفأ بيروت"),
            ("2025-08-15", "عيد انتقال العذراء"),
            ("2025-09-04", "ذكرى المولد النبوي الشريف (تقديري)"),
            ("2025-11-22", "ذكرى الاستقلال"),
            ("2025-12-25", "عيد الميلاد"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-06", "Armenian Orthodox Christmas Day"),
            ("2025-02-09", "Saint Maron's Day"),
            ("2025-02-14", "Rafik Hariri Memorial Day"),
            ("2025-03-25", "Feast of the Annunciation"),
            ("2025-03-30", "Eid al-Fitr"),
            ("2025-03-31", "Eid al-Fitr"),
            ("2025-04-18", "Catholic Good Friday; Orthodox Good Friday"),
            ("2025-04-19", "Orthodox Holy Saturday"),
            ("2025-04-21", "Catholic Easter Monday; Orthodox Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-04", "Martyrs' Day"),
            ("2025-05-11", "Resistance and Liberation Day"),
            ("2025-06-06", "Eid al-Adha"),
            ("2025-06-07", "Eid al-Adha"),
            ("2025-06-26", "Islamic New Year"),
            ("2025-07-05", "Ashura (estimated)"),
            ("2025-08-04", "Anniversary of the tragedy of Beirut port explosion"),
            ("2025-08-15", "Assumption Day"),
            ("2025-09-04", "Prophet's Birthday (estimated)"),
            ("2025-11-22", "Independence Day"),
            ("2025-12-25", "Christmas Day"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2025-01-01", "Nouvel An"),
            ("2025-01-06", "Noël Arménien"),
            ("2025-02-09", "Saint Maron"),
            ("2025-02-14", "Commémoration de l'assassinat du PM Rafic Hariri"),
            ("2025-03-25", "Annonciation"),
            ("2025-03-30", "Eid El Fitr"),
            ("2025-03-31", "Eid El Fitr"),
            ("2025-04-18", "Vendredi Saint Catholique; Vendredi Saint Orthodoxe"),
            ("2025-04-19", "Samedi Saint Orthodoxe"),
            ("2025-04-21", "Lundi de Pâques Catholique; Lundi de Pâques Orthodoxe"),
            ("2025-05-01", "Fête du Travail"),
            ("2025-05-04", "Journée des Martyrs"),
            ("2025-05-11", "Résistance et Libération"),
            ("2025-06-06", "Adha"),
            ("2025-06-07", "Adha"),
            ("2025-06-26", "Hégire"),
            ("2025-07-05", "Achoura (estimé)"),
            ("2025-08-04", "Explosion du port de Beyrouth"),
            ("2025-08-15", "Assomption"),
            ("2025-09-04", "Naissance du Prophète (estimé)"),
            ("2025-11-22", "Fête Nationale"),
            ("2025-12-25", "Noël"),
        )
